"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.importHandler = importHandler;
const path_1 = require("path");
const git_utils_1 = require("../../utils/git-utils");
const fs_1 = require("fs");
const util_1 = require("util");
const tmp_1 = require("tmp");
const enquirer_1 = require("enquirer");
const output_1 = require("../../utils/output");
const createSpinner = require("ora");
const init_v2_1 = require("../init/init-v2");
const nx_json_1 = require("../../config/nx-json");
const workspace_root_1 = require("../../utils/workspace-root");
const package_manager_1 = require("../../utils/package-manager");
const rmAsync = (0, util_1.promisify)(fs_1.rm);
const copyFileAsync = (0, util_1.promisify)(fs_1.copyFile);
const mkdirAsync = (0, util_1.promisify)(fs_1.mkdir);
function wait(ms) {
    return new Promise((resolve) => setTimeout(resolve, ms));
}
const importRemoteName = '__tmp_nx_import__';
const tempImportBranch = '__tmp_import_stage__';
const tempFileDir = '__tmp_import_stage__';
async function prepareSourceRepo(gitClient, ref, source, destination) {
    const spinner = createSpinner(`Checking out a temporary branch, ${tempImportBranch} based on ${ref}`).start();
    await gitClient.checkout(tempImportBranch, {
        new: true,
        base: `origin/${ref}`,
    });
    spinner.succeed(`Created a ${tempImportBranch} branch based on ${ref}`);
    const relativeSourceDir = (0, path_1.relative)(gitClient.root, (0, path_1.join)(gitClient.root, source));
    const destinationInSource = (0, path_1.join)(gitClient.root, destination);
    spinner.start(`Moving files and git history to ${destinationInSource}`);
    if (relativeSourceDir === '') {
        const files = await gitClient.getGitFiles('.');
        try {
            await rmAsync(destinationInSource, {
                recursive: true,
            });
        }
        catch { }
        await mkdirAsync(destinationInSource, { recursive: true });
        const gitignores = new Set();
        for (const file of files) {
            if ((0, path_1.basename)(file) === '.gitignore') {
                gitignores.add(file);
                continue;
            }
            const newPath = (0, path_1.join)(destinationInSource, file);
            await mkdirAsync((0, path_1.dirname)(newPath), { recursive: true });
            try {
                await gitClient.move(file, newPath);
            }
            catch {
                await wait(100);
                await gitClient.move(file, newPath);
            }
        }
        await gitClient.commit('chore(repo): prepare for import');
        for (const gitignore of gitignores) {
            await gitClient.move(gitignore, (0, path_1.join)(destinationInSource, gitignore));
        }
        await gitClient.amendCommit();
        for (const gitignore of gitignores) {
            await copyFileAsync((0, path_1.join)(destinationInSource, gitignore), (0, path_1.join)(gitClient.root, gitignore));
        }
    }
    else {
        let needsSquash = false;
        try {
            await rmAsync(destinationInSource, {
                recursive: true,
            });
            await gitClient.commit('chore(repo): prepare for import');
            needsSquash = true;
        }
        catch { }
        await mkdirAsync(destinationInSource, { recursive: true });
        const files = await gitClient.getGitFiles('.');
        for (const file of files) {
            if (file === '.gitignore') {
                continue;
            }
            if (!(0, path_1.relative)(source, file).startsWith('..')) {
                const newPath = (0, path_1.join)(destinationInSource, file);
                await mkdirAsync((0, path_1.dirname)(newPath), { recursive: true });
                try {
                    await gitClient.move(file, newPath);
                }
                catch {
                    await wait(100);
                    await gitClient.move(file, newPath);
                }
            }
            await rmAsync((0, path_1.join)(gitClient.root, file), {
                recursive: true,
            });
        }
        await gitClient.commit('chore(repo): prepare for import 2');
        if (needsSquash) {
            await gitClient.squashLastTwoCommits();
        }
    }
    spinner.succeed(`Prepared for import`);
}
async function confirmOrExitWithAnError(message) {
    const { confirm } = await (0, enquirer_1.prompt)([
        {
            type: 'select',
            name: 'confirm',
            choices: ['Yes', 'No'],
            message,
        },
    ]);
    if (confirm === false) {
        throw new Error('Cancelled');
    }
}
async function mergeRemoteSource(destinationGitClient, sourceRemoteUrl, branch, destination) {
    const spinner = createSpinner();
    spinner.start(`Merged ${branch} from ${sourceRemoteUrl} into ${destination}`);
    spinner.start(`Adding ${sourceRemoteUrl} as a remote`);
    await createTemporarySourceRemote(destinationGitClient, sourceRemoteUrl, importRemoteName);
    spinner.succeed(`Added ${sourceRemoteUrl} as a remote`);
    spinner.start(`Merging files and git history from ${branch} from ${sourceRemoteUrl} into ${destination}`);
    await destinationGitClient.mergeUnrelatedHistories(`${importRemoteName}/${branch}`, `feat(repo): merge ${sourceRemoteUrl}`);
    spinner.succeed(`Merged files and git history from ${branch} from ${sourceRemoteUrl} into ${destination}`);
}
async function createTemporarySourceRemote(destinationGitClient, sourceRemoteUrl, remoteName) {
    try {
        await destinationGitClient.deleteGitRemote(remoteName);
    }
    catch { }
    await destinationGitClient.addGitRemote(remoteName, sourceRemoteUrl);
    await destinationGitClient.fetch(remoteName);
}
async function importHandler(options) {
    let { sourceRemoteUrl, ref, source, destination } = options;
    output_1.output.log({
        title: 'Nx will walk you through the process of importing another repository into the workspace:',
        bodyLines: [
            `1. Nx will clone another repository into a temporary directory`,
            `2. Code to be imported will be moved to the same directory it will be imported into`,
            `3. A temporary branch will be pushed to the remote repository`,
            `4. The code will be merged into this workspace`,
            `5. Nx will recommend plugins to integrate tools used in the import code with Nx`,
            '',
            `Git history will be preserved`,
        ],
    });
    const tempRepoPath = (0, path_1.join)(tmp_1.tmpdir, 'nx-import');
    if (!sourceRemoteUrl) {
        sourceRemoteUrl = (await (0, enquirer_1.prompt)([
            {
                type: 'input',
                name: 'sourceRemoteUrl',
                message: 'What is the Remote URL of the repository you want to import?',
                required: true,
            },
        ])).sourceRemoteUrl;
    }
    const spinner = createSpinner(`Cloning ${sourceRemoteUrl} into ${tempRepoPath}`).start();
    try {
        await rmAsync(tempRepoPath, { recursive: true });
    }
    catch { }
    await mkdirAsync(tempRepoPath, { recursive: true });
    const sourceGitClient = await (0, git_utils_1.cloneFromUpstream)(sourceRemoteUrl, (0, path_1.join)(tempRepoPath, 'repo'));
    spinner.succeed(`Cloned into ${tempRepoPath}`);
    if (!ref) {
        const branchChoices = await sourceGitClient.listBranches();
        ref = (await (0, enquirer_1.prompt)([
            {
                type: 'autocomplete',
                name: 'ref',
                message: `Which branch do you want to import?`,
                choices: branchChoices,
                required: true,
            },
        ])).ref;
    }
    await wait(100);
    if (!source) {
        source = (await (0, enquirer_1.prompt)([
            {
                type: 'input',
                name: 'source',
                message: `Which directory do you want to import into this workspace? (leave blank to import the entire repository)`,
            },
        ])).source;
    }
    if (!destination) {
        destination = (await (0, enquirer_1.prompt)([
            {
                type: 'input',
                name: 'destination',
                message: 'Where in this workspace should the code be imported into?',
                required: true,
            },
        ])).destination;
    }
    await prepareSourceRepo(sourceGitClient, ref, source, destination);
    console.log(await sourceGitClient.showStat());
    output_1.output.log({
        title: `${sourceRemoteUrl} has been prepared to be imported into this repo on a temporary branch: ${tempImportBranch}`,
    });
    await confirmOrExitWithAnError(`Push ${tempImportBranch} to ${sourceRemoteUrl} (git push -u -f origin ${tempImportBranch})\nAnd then import it from there into ${destination} in this workspace?`);
    await sourceGitClient.push(tempImportBranch);
    // Ready to import
    const destinationGitClient = new git_utils_1.GitRepository(process.cwd());
    const mergeSpinner = createSpinner().start(`Importing `);
    await mergeRemoteSource(destinationGitClient, sourceRemoteUrl, tempImportBranch, destination);
    mergeSpinner.succeed(`Imported ${source} from ${sourceRemoteUrl} into ${destination}!`);
    const pmc = (0, package_manager_1.getPackageManagerCommand)();
    const nxJson = (0, nx_json_1.readNxJson)(workspace_root_1.workspaceRoot);
    const { plugins, updatePackageScripts } = await (0, init_v2_1.detectPlugins)(nxJson);
    if (plugins.length > 0) {
        output_1.output.log({ title: 'Installing Plugins' });
        (0, init_v2_1.installPlugins)(workspace_root_1.workspaceRoot, plugins, pmc, updatePackageScripts);
        await destinationGitClient.amendCommit();
    }
    console.log(await destinationGitClient.showStat());
    output_1.output.log({
        title: `Merging these changes`,
        bodyLines: [
            `MERGE these changes when merging`,
            `Do NOT squash and do NOT rebase these changes when merging these changes.`,
            `If you would like to UNDO these changes, run "git reset HEAD~1 --hard"`,
        ],
    });
}
